<?php

// @codingStandardsIgnoreStart
class ControllerExtensionModuleRoistat extends Controller {

    public function index() {
        $this->load->model('setting/setting');
        $settings = $this->model_setting_setting->getSetting('module_roistat');

        if (!array_key_exists('module_roistat_password', $settings) || !array_key_exists('module_roistat_username', $settings)) {
            echo 'Data of authorization aren\'t set';
            exit;
        }

        $action_method = $this->_getRequest('action');
        $auth_token    = $this->_getRequest('user_token');

        $username = $settings['module_roistat_username'];
        $password = $settings['module_roistat_password'];

        if ((string)$auth_token !== (string)md5($username . $password)) {
            echo 'Incorrect token';
            exit;
        }

        switch ($action_method) {
            case 'import_scheme':
                $output = $this->_loadImportScheme();
                break;
            case 'export':
                $output = $this->_loadOrders();
                break;
            case 'export_clients':
                $output = $this->_loadClients();
                break;
            case 'lead':
                $output = $this->_createLead();
                break;
            default:
                $output = array('status' => 'error', 'message' => 'Action is undefined');
        }

        $this->response->addHeader('Content-Type: application/json; charset=utf8');
        $this->response->setOutput(json_encode($output));
    }

    /**
     * @return array
     */
    private function _loadImportScheme() {
        return array(
            'statuses' => $this->_loadStatuses(),
            'fields'   => $this->_loadFields(),
            'managers' => $this->_loadManagers(),
        );
    }

    /**
     * @return array
     */
    private function _loadStatuses() {
        $statusesForOutput = array();

        $language_id = $this->config->get('config_language_id');

        $statuses = $this->db->query("SELECT * FROM `" . DB_PREFIX . "order_status` WHERE `language_id` = '" . $language_id . "';");
        if (!is_array($statuses->rows)) {
            return $statusesForOutput;
        }

        foreach ($statuses->rows as $status) {
            $statusesForOutput[] = array(
                'id'   => $status['order_status_id'],
                'name' => $status['name'],
            );
        }
        return $statusesForOutput;
    }

    /**
     * @return array
     */
    private function _loadFields() {
        $fieldsForOutput = array();

        $fields = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "order`;");
        if (!is_array($fields->rows)) {
            return $fieldsForOutput;
        }

        foreach ($fields->rows as $field_id => $field) {
            $fieldsForOutput[] = array(
                'id'   => $field_id + 1,
                'name' => $field['Field'],
            );
        }
        return $fieldsForOutput;
    }

    /**
     * @return array
     */
    private function _loadManagers() {
        $managersForOutput = array();

        $managers  = $this->db->query("SELECT * FROM `" . DB_PREFIX . "user`;");
        if (!is_array($managers->rows)) {
            return $managersForOutput;
        }

        foreach ($managers->rows as $manager) {
            $managersForOutput[] = array(
                'id'    => $manager['user_id'],
                'name'  => $manager['username'],
                'email' => $manager['email'],
            );
        }
        return $managersForOutput;
    }

    /**
     * @return array
     */
    private function _loadOrders() {
        $modifed_date = (int)$this->_getRequest('date', time() - 31*24*60*60);
        $store_id     = $this->config->get('config_store_id');

        $sql = "SELECT * FROM `" . DB_PREFIX . "order`
                WHERE `store_id` = '" . $store_id . "'
                AND `date_modified` > {$modifed_date};";

        $statuses = $this->_loadStatuses();

        $orders = $this->db->query($sql);
        if (!is_array($orders->rows)) {
            return array('orders' => array(), 'statuses' => $statuses);
        }

        $systemFields = array('order_id', 'date_added', 'order_status_id', 'total', 'roistat_visit', 'customer_id', 'shipping_method', 'payment_method', 'affiliate_id');

        $ordersForOutput = array();
        foreach ($orders->rows as $order) {
            $additionalFields = array();
            foreach ($order as $field => $value) {
                if (!in_array($field, $systemFields)) {
                    $additionalFields[$field] = $value;
                }
            }

            $fieldCost = array_key_exists('cost', $additionalFields) && is_scalar($additionalFields['cost'])
                ? $additionalFields['cost']
                : 0;

            $newOrder = array(
                'id'          => $order['order_id'],
                'date_create' => $order['date_added'],
                'status'      => $order['order_status_id'],
                'price'       => $order['total'],
                'cost'        => $fieldCost,
                'visit'       => $order['roistat_visit'],
                'client_id'   => $order['customer_id'],
                'fields'      => $additionalFields,
            );
            if (array_key_exists('shipping_method', $order) && $order['shipping_method'] !== '') {
                $newOrder['fields'][$this->language->get('entry_shipping_method')] = $order['shipping_method'];
            }
            if (array_key_exists('payment_method', $order) && $order['payment_method'] !== '') {
                $newOrder['fields'][$this->language->get('entry_payment_method')] = $order['payment_method'];
            }
            if (array_key_exists('affiliate_id', $order) && $order['affiliate_id'] !== '') {
                $newOrder['fields'][$this->language->get('text_affiliate')] = $order['affiliate_id'];
            }
            $ordersForOutput[] = $newOrder;
        }
        return array('orders' => $ordersForOutput, 'statuses' => $statuses);
    }

    /**
     * @return array
     */
    private function _createLead() {
        $customer_id = 0;
        $store_id    = $this->config->get('config_store_id');
        $title       = $this->_getRequest('title');
        $name        = $this->_getRequest('name');
        $email       = $this->_getRequest('email');
        $phone       = $this->_getRequest('phone');
        $visit       = $this->_getRequest('visit');
        $comment     = $this->_getRequest('text');
        $first_name  = ($name === '' || $name === null) ? $title : $name;

        $found_customer_id = $this->_getCustomerIdByEmail($email);
        $customer_id = $found_customer_id === null ? $this->_addClient($first_name, $email, $phone, $store_id) : $found_customer_id;

        $leadId = $this->_storeLead($title, $first_name, $email, $phone, $visit, $comment, $customer_id, $store_id);
        return array('status' => 'ok', 'order_id' => $leadId);
    }

    /**
     * @param string $email
     * @return array
     */
    private function _getCustomerIdByEmail($email) {
        $query = $this->db->query("SELECT DISTINCT * FROM " . DB_PREFIX . "customer WHERE LCASE(email) = '{$this->db->escape($email)}';");
        return count($query->row) > 0 ? $query->row['customer_id'] : null;
    }

    /**
     * @param $title
     * @param $first_name
     * @param $email
     * @param $phone
     * @param $visit
     * @param $comment
     * @param $customer_id
     * @param $store_id
     * @return int
     */
    private function _storeLead($title, $first_name, $email, $phone, $visit, $comment, $customer_id, $store_id) {
        $store_name      = $this->config->get('config_name');
        $store_url       = $this->config->get('config_url');
        $language_id     = $this->config->get('config_language_id');
        $currency        = $this->config->get('config_currency');
        $order_status_id = $this->config->get('config_order_status_id');

        $sql = "INSERT INTO `" . DB_PREFIX . "order`
                        SET `order_status_id` = '{$order_status_id}',
                            `language_id` = '{$language_id}',
                            `telephone` = '{$this->db->escape($phone)}',
                            `email` = '{$this->db->escape($email)}',
                            `firstname` = '{$this->db->escape($first_name)}',
                            `customer_id` = '{$customer_id}',
                            `store_url` = '{$store_url}',
                            `store_name` = '{$this->db->escape($title)} ({$store_name})',
                            `store_id` = '{$store_id}',
                            `comment` = '{$this->db->escape($comment)}',
                            `roistat_visit` = '{$this->db->escape($visit)}',
                            `payment_code` = 'cod',
                            `currency_code` = '{$currency}',
                            `date_added` = NOW(),
                            `date_modified` = NOW()";
        $this->db->query($sql);
        return $this->db->getLastId();
    }

    /**
     * @return array
     */
    private function _loadClients() {
        $date = (int)$this->_getRequest('date', time() - 31*24*60*60);
        $store_id = $this->config->get('config_store_id');

        $customerCompanies = $this->getCustomerCompanies();
        $sql = "SELECT `customer_id`, `firstname`, `lastname`, `email`, `telephone`
                FROM `" . DB_PREFIX . "customer`
                WHERE `store_id` = '" . $store_id . "'
                AND `date_added` > {$date};";

        $clientsForOutput = array();

        $clients = $this->db->query($sql);
        if (!is_array($clients->rows)) {
            return array('clients' => $clientsForOutput);
        }

        foreach ($clients->rows as $client) {
            $clientName = $client['firstname'] . $client['lastname'];

            $newClient  = array(
                'id'        => $client['customer_id'],
                'name'      => $clientName,
                'phone'     => $client['telephone'],
                'email'     => $client['email'],
            );

            if (array_key_exists($client['customer_id'], $customerCompanies['companies'])
                && $customerCompanies['companies'][$client['customer_id']] !== '') {
                $newClient['company'] = $customerCompanies['companies'][$client['customer_id']];
            }
            $clientsForOutput[] = $newClient;
        }
        return array('clients' => $clientsForOutput);
    }

    /**
     * @return array
     */
    private function getCustomerCompanies() {
        $companiesForOutput = array();

        $companies = $this->db->query("SELECT `customer_id`, `company` FROM `" . DB_PREFIX . "customer_affiliate`;");
        if (!is_array($companies->rows)) {
            return array('companies' => $companiesForOutput);
        }

        foreach ($companies->rows as $company) {
            $companiesForOutput[$company['customer_id']] = $company['company'];
        }
        return array('companies' => $companiesForOutput);
    }

    /**
     * @param string $name
     * @param string $email
     * @param string $phone
     * @return int
     */
    private function _addClient($name, $email, $phone, $store_id) {
        $sql = "INSERT INTO `" . DB_PREFIX . "customer`
                        SET `firstname` = '{$this->db->escape($name)}',
                            `telephone` = '{$this->db->escape($phone)}',
                            `email` = '{$this->db->escape($email)}',
                            `customer_group_id` = '1',
                            `status` = '1',
                            `store_id` = '{$store_id}',
                            `date_added` = NOW()";
        $this->db->query($sql);
        return $this->db->getLastId();
    }

    /**
     * @param string $key
     * @param mixed $default
     * @return string
     */
    private function _getRequest($key, $default = null) {
        return array_key_exists($key, $this->request->request) ? $this->request->request[$key] : $default;
    }

    /**
     * For AJAX in order_form
     */
    public function updateVisit() {
        $this->load->language('module/roistat');

        if ($this->_isAdmin()) {
            $order_id = $this->_getRequest('order_id');
            $visit_id = $this->_getRequest('visit');
            $this->_updateVisitByOrderId($order_id, $visit_id);

            $output = array('success' => $this->language->get('text_edit_visit_success'));
        } else {
            $output = array('error' => $this->language->get('text_edit_visit_error'));
        }
        $this->response->addHeader('Content-Type: application/json; charset=utf8');
        $this->response->setOutput(json_encode($output));
    }

    /**
     * @param int $order_id
     * @param string $visit
     */
    private function _updateVisitByOrderId($order_id, $visit) {
        $this->db->query("UPDATE `" . DB_PREFIX . "order` SET `roistat_visit` = '" . $visit . "' WHERE `order_id` = '" . (int)$order_id . "';");
    }

    /**
     * @return bool
     */
    private function _isAdmin() {
        $user = new User($this->registry);
        return (bool)$user->isLogged();
    }
}